package	jp.co.sds_corp.util.daemon.file;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.config.UnspecifiedPropertyException;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* プロセスを起動します。
<P>
XMLは以下の形式です。
<PRE>
&lt;Service name="Test" class="jp.co.sds_corp.util.daemon.file.ExecuteService"&gt;
	&lt;dir&gt;.&lt;/dir&gt;
	&lt;command&gt;ls -ltr&lt;/command&gt;
	&lt;env name="LANG" value="C"/&gt;
&lt;/Service&gt;
</PRE>

&lt;command&gt;要素は複数指定することが可能です。
&lt;env&gt;要素は複数指定することが可能です。
また、&lt;dir&gt;要素、&lt;env&gt;要素は省略可能です。
<P>
*/
public class ExecuteService extends AbstractService
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String		dir;
	HashMap<String,String>	mapEnv = new HashMap<String,String>();
	Vector<String>		vecCmd = new Vector<String>();

	/**
	* このクラスを設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		Iterator<IConfiguration> itr = config.getChilds( "command" );
		if ( itr == null ) {
			throw( new UnspecifiedPropertyException( "<command> not exist" ));
		}
		while ( itr.hasNext()) {
			IConfiguration cfg = (IConfiguration )itr.next();
			vecCmd.add( cfg.get() );
		}
		try {
			dir = config.get( "dir" );
		} catch ( Exception ex ) {
		}

		itr = config.getChilds( "env" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			String name = c.getAttributes().get( "name" );
			String value = c.getAttributes().get( "value" );
			mapEnv.put( name, value );
		}
	}

	/**
	* 子プロセスの起動処理を開始します。
	*/
	public void start() {
		Status stat = Status.WAIT;
		setStatus( Status.RUN );
		try {
			File curd = null;
			if ( dir != null ) {
				curd = new File( dir );
			}
			for ( int i = 0; i < vecCmd.size(); i++ ) {
				String cmd = (String )vecCmd.get( i );
				int nRet = execChild( cmd, curd );
				if ( isDebug()) {
					logger.debug( this, "exit=" + nRet );
				}
				if ( nRet != 0 ) {
					stat = Status.WARN;
				}
			}
		} catch ( Throwable ex ) {
			logger.error( this, "command execute error", ex );
			setStatus( Status.ERROR );
			return;
		}
		setStatus( stat );
	}

	private class InputReader implements Runnable
	{
		String	name;
		String	symbol;
		InputStream is;

		InputReader( String name, String symbol, InputStream is ) {
			this.name = name;
			this.symbol = symbol;
			this.is = is;
		}
		public void run() {
			try {
				BufferedReader rd = new BufferedReader(
									new InputStreamReader( is ));
				for ( ;; ) {
					String line = rd.readLine();
					if ( line == null ) {
						break;
					}
					if ( isDebug()) {
						logger.debug( this, name + ":" + symbol + ": " + line );
					}
				}
			} catch ( Exception ex ) {
				logger.error( this, "stream read error", ex );
			}
		}
	}

	/**
	* 子プロセスを起動します。
	*/
	protected int execChild( String cmd, File curd ) throws Exception {
		if ( isDebug()) {
			logger.debug( this, "execute [" + cmd + "]" );
		}
		String[] aEnv;
		aEnv = new String[mapEnv.size()];
		int i = 0;
		Iterator<String> itr = mapEnv.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			aEnv[i] = key + "=" + mapEnv.get( key );
			i++;
		}

		Process proc = Runtime.getRuntime().exec( cmd, aEnv, curd );
		InputReader or = new InputReader( cmd, "S", proc.getInputStream());
		Thread stdout = new Thread( or );
		stdout.start();
		InputReader er = new InputReader( cmd, "E", proc.getErrorStream());
		Thread stderr = new Thread( er );
		stderr.start();
		int nRet = proc.waitFor();
		return nRet;
	}

	/**
	* ファイルの受信処理を終了します。
	*/
	public void stop() {
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		if ( dir != null ) {
			config.addChild( "dir", dir );
		}
		for ( int i = 0; i < vecCmd.size(); i++ ) {
			String cmd = (String )vecCmd.get( i );
			config.addChild( "command", cmd );
		}
		Iterator<String> itr = mapEnv.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			IConfiguration c = config.addChild( "env" );
			c.setAttribute( "name", key );
			c.setAttribute( "value", (String )mapEnv.get( key ));
		}
	}

	/**
	* このサービスの文字列表現を返します。
	*/
	public String toString() {
		String ret = super.toString();
		ret += " dir=[" + dir + "] env=[";
		Iterator<String> itr = mapEnv.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			ret += key + "=" + mapEnv.get( key ) + ";";
		}
		ret += "]";
		for ( int i = 0; i < vecCmd.size(); i++ ) {
			String cmd = (String )vecCmd.get( i );
			ret += " cmd=[" + cmd + "]";
		}
		return ret;
	}
}
