package jp.co.sds_corp.util.remote;

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* プロセスコントローラの実装を提供します。
* このクラスでは認証機能のみを提供しますので、個別のコマンドはこのクラスを
* 継承したクラスで実装する必要があります。
*/
public class ProcessRemoteControlImpl extends UnicastRemoteObject
            implements ProcessRemoteControl
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	Registry		reg;
	ServerProcess	server;
	String			name;
	int				port;
	String			url;

	/**
	* localhostのRMI Registryからサーバ実装を取得し、
	* プロセスコントローラを初期化します。
	* @param	server	プロセスコントローラのサーバ実装
	* @param	port	RMI Registry のポート番号
	* @param	name	RMI Registry の登録名
	*/
	public ProcessRemoteControlImpl( ServerProcess server, int port, String name )
		throws RemoteException, MalformedURLException, AlreadyBoundException
	{
		this( server, "localhost", port, name );
	}

	/**
	* 指定されたホストのRMI Registryからサーバ実装を取得し、
	* プロセスコントローラを初期化します。
	* @param	server	プロセスコントローラのサーバ実装
	* @param	host	RMI Registry のホスト名
	* @param	port	RMI Registry のポート番号
	* @param	name	RMI Registry の登録名
	*/
	public ProcessRemoteControlImpl( ServerProcess server,
			String host, int port, String name )
		throws RemoteException, MalformedURLException, AlreadyBoundException
	{
		this( server, host, port, name, true );
	}

	/**
	* 指定されたホストのRMI Registryからサーバ実装を取得し、
	* プロセスコントローラを初期化します。
	* @param	server	プロセスコントローラのサーバ実装
	* @param	host	RMI Registry のホスト名
	* @param	port	RMI Registry のポート番号
	* @param	name	RMI Registry の登録名
	* @param	bFlg	RMI Registry をCreateするかどうかのフラグ
	*/
	public ProcessRemoteControlImpl( ServerProcess server,
			String host, int port, String name, boolean bFlg )
		throws RemoteException, MalformedURLException, AlreadyBoundException
	{
		this.server = server;
		this.name = name;
		if ( port > 0 ) {
			this.port = port;
		} else {
			this.port = Registry.REGISTRY_PORT;
		}

		if ( bFlg ) {
			reg = LocateRegistry.createRegistry( port );
		} else {
			reg = LocateRegistry.getRegistry( host, port );
		}
		url = "rmi://" + host + ":" + port + "/" + name;
		Naming.bind( url, this );
		logger.debug( this, "Process remote control object binded."
							+ " url=" + url + " " + bFlg );
	}

	/**
	* RMI Registry から名前を削除します。
	* @param	name	RMI Registry の登録名
	*/
	public void unbind( String name )
		throws RemoteException, NotBoundException
	{
		if ( reg != null ) {
			reg.unbind( name );
			// logger.debug( "unbind " + url );
			// reg.unbind( url );
		}
	}

	private boolean allowRemoteCall = true;
	/**
	* RMI認証を行ないます。
	* @param	key		RMI認証キー
	*/
	public AuthKey auth(AuthKey key) throws RemoteException {
		try {
			InetAddress client = InetAddress.getByName(getClientHost());
			// ローカルホストからのリクエストは取り合えず素通し
			if (!InetAddress.getLocalHost().equals(client)) {
				if (!allowRemoteCall) {
					// 例外クラス作るのサボってる
					throw new RemoteException("access denied");
				}
				AuthUtility.isValidUser(key);
			}
			return AuthUtility.createAuthenticatedKey(key);
		} catch (Exception e) {
			// ServerNotActiveException/UnknownHostException
			throw new AuthenticationFailedException();
		}
	}

	/**
	* プロセスコントローラのサーバ実装を返します。
	* @return	プロセスコントローラのサーバ実装
	*/
	public ServerProcess getServer() {
		return server;
	}
}

